#include <windows.h>
#include <streams.h>
#include <dmodshow.h>
#include <dmoreg.h>
#include <wmcodecids.h>
#include <cs.h>
#include <csmedia.h>

#include "utils.h"

HRESULT GetCameraDriver(TCHAR* name)
{
	HANDLE	                    handle=NULL;
	DEVMGR_DEVICE_INFORMATION   di;
	GUID                        guidCamera={ 0xCB998A05, 0x122C, 0x4166, 0x84, 0x6A, 0x93, 0x3E, 0x4D, 0x7E, 0x3C, 0x86 };
	// Note about the above: The driver material doesn't ship as part of the SDK. This GUID is hardcoded
	// here to be able to enumerate the camera drivers and pass the name of the driver to the video capture filter

	if(!name)
		return E_POINTER;

	di.dwSize=sizeof(di);

	handle=FindFirstDevice(DeviceSearchByGuid,&guidCamera,&di);
	
    if((handle==NULL)||(di.hDevice==NULL)||(handle==INVALID_HANDLE_VALUE))
	{
        if (handle)
    	    FindClose(handle);
		return HRESULT_FROM_WIN32(GetLastError());
	}

	StringCchCopy(name,MAX_PATH,di.szLegacyName);
	return S_OK;
}


BOOL ConfigComparePrint(AM_MEDIA_TYPE* config,void* parm)
{
    TCHAR*  subtype=TEXT("");

    if(IsEqualGUID(config->subtype, MEDIASUBTYPE_YUY2)) 
	{
		subtype=TEXT("YUY2");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_RGB565)) 
	{
		subtype=TEXT("RGB565");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_RGB24)) 
	{
		subtype=TEXT("RGB24");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_UYVY)) 
	{
		subtype=TEXT("UYVY");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_YUYV)) 
	{
		subtype=TEXT("YUYV");		
	}
    else
    {   
        _tprintf(TEXT("Invalid media subtype\r\n"));
        return FALSE;
    }

    DWORD videoWidth=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biWidth);
    DWORD videoHeight=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biHeight);

    _tprintf(TEXT("%s@%dx%d\r\n"),subtype,videoWidth,videoHeight);
    return FALSE;    
}

BOOL ConfigCompareString(AM_MEDIA_TYPE* config,void* parm)
{
    TCHAR   resstring[1024];
    TCHAR*  subtype=TEXT("");

    if(IsEqualGUID(config->subtype, MEDIASUBTYPE_YUY2)) 
	{
		subtype=TEXT("YUY2");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_RGB565)) 
	{
		subtype=TEXT("RGB565");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_RGB24)) 
	{
		subtype=TEXT("RGB24");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_UYVY)) 
	{
		subtype=TEXT("UYVY");		
	}
	else if(IsEqualGUID(config->subtype, MEDIASUBTYPE_YUYV)) 
	{
		subtype=TEXT("YUYV");		
	}

    LONG videoWidth=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biWidth);
    LONG videoHeight=abs(((VIDEOINFOHEADER*)config->pbFormat)->bmiHeader.biHeight);

    _stprintf_s(resstring,1023,TEXT("%s@%dx%d"),subtype,videoWidth,videoHeight);
    
    // we are just printing out the help message
    if (_tcsicmp((const wchar_t*)parm,resstring)==0)
        return TRUE;
    return FALSE;
}

HRESULT EnumConfigs(IAMStreamConfig* pCaptureStreamConfig,AM_MEDIA_TYPE** retconfig,ConfigCompareFn* comparefn,void* parm)
{
    HRESULT hr=S_OK;
    int     rescount,structsize;

    if (retconfig)
        *retconfig=NULL;

    if (FAILED(hr=pCaptureStreamConfig->GetNumberOfCapabilities(&rescount,&structsize)))
        return hr;

    // checks formats until it finds one that matches the command line
    AM_MEDIA_TYPE* config=NULL;

    for (int i=0;i<rescount;i++)
    {
	    VIDEO_STREAM_CONFIG_CAPS scc;

        if (FAILED(hr=pCaptureStreamConfig->GetStreamCaps(i,&config,(BYTE*)&scc)))
            return hr;

        // we found the right resolution
        if ((*comparefn)(config,parm))
        {
            if (retconfig)
                *retconfig=config;
            return S_OK;
        }

        DeleteMediaType(config);
    }    
    // enumeration worked, pointer may be still NULL
    return S_OK;
}